#!/usr/bin/env python3
"""
NeuroUO Simple Launcher
Downloads and launches ClassicUO configured for NeuroUO server
"""
import os
import sys
import json
import urllib.request
import zipfile
import subprocess
import tempfile
from pathlib import Path

SERVER_IP = "95.216.11.178"
SERVER_PORT = 2593

# Try multiple ClassicUO download sources
CLASSICUO_URLS = [
    "https://github.com/ClassicUO/ClassicUO/releases/download/0.1.11.7/ClassicUO-0.1.11.7-win-x64.zip",
    "https://github.com/bittiez/ClassicUO/releases/download/v3.22.1/ClassicUO-Bittiez-Release.zip",
    "https://github.com/kaczy93/centreduo/releases/download/1.0.1.1/CentrED.UO_1.0.1.1_win-x64.zip"
]

UO_CLIENT_URL = "https://neurouo.fun/downloads/UOClient-7.0.20.0.zip"

def download_file(url, dest_path):
    """Download file with progress"""
    print(f"Downloading from {url}...")
    try:
        # Add headers to avoid 403 errors
        req = urllib.request.Request(url, headers={
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'
        })
        
        with urllib.request.urlopen(req) as response:
            total_size = int(response.headers.get('Content-Length', 0))
            downloaded = 0
            
            with open(dest_path, 'wb') as f:
                while True:
                    chunk = response.read(8192)
                    if not chunk:
                        break
                    f.write(chunk)
                    downloaded += len(chunk)
                    
                    if total_size > 0:
                        percent = (downloaded / total_size) * 100
                        print(f"\rProgress: {percent:.1f}%", end='', flush=True)
            
            print("\nDownload complete!")
            return True
            
    except Exception as e:
        print(f"\nError downloading: {e}")
        return False

def extract_zip(zip_path, dest_path):
    """Extract zip file"""
    print(f"Extracting to {dest_path}...")
    os.makedirs(dest_path, exist_ok=True)
    
    with zipfile.ZipFile(zip_path, 'r') as zip_ref:
        zip_ref.extractall(dest_path)
    
    os.remove(zip_path)
    print("Extraction complete!")

def find_classicuo_exe(base_path):
    """Find ClassicUO.exe in extracted folder"""
    for root, dirs, files in os.walk(base_path):
        for file in files:
            if file.lower() == "classicuo.exe":
                return Path(root) / file
    return None

def create_settings(classicuo_dir, client_path):
    """Create settings.json for ClassicUO"""
    settings = {
        "username": "",
        "password": "",
        "ip": SERVER_IP,
        "port": SERVER_PORT,
        "ultimaonlinedirectory": str(client_path),
        "profilespath": "",
        "clientversion": "",
        "lang": "ENU",
        "lastservernum": 1,
        "last_server_name": "NeuroUO",
        "fps": 60,
        "window_position": None,
        "window_size": None,
        "is_win_maximized": False,
        "saveaccount": True,
        "autologin": False,
        "reconnect": True,
        "reconnect_time": 10,
        "login_music": True,
        "login_music_volume": 70,
        "shard_type": 0,
        "fixed_time_step": True,
        "run_mouse_in_separate_thread": True,
        "force_driver": 0,
        "use_verdata": False,
        "maps_layouts": "",
        "encryption": 0,
        "plugins": []
    }
    
    settings_path = classicuo_dir / "settings.json"
    with open(settings_path, 'w') as f:
        json.dump(settings, f, indent=2)
    
    print(f"Created settings at: {settings_path}")

def create_batch_launcher(launcher_dir, classicuo_exe, client_path):
    """Create a batch file launcher"""
    batch_content = f"""@echo off
title NeuroUO Launcher
echo ==========================================
echo           NeuroUO Client Launcher
echo ==========================================
echo.
echo Server: {SERVER_IP}:{SERVER_PORT}
echo.

cd /d "{classicuo_exe.parent}"
start "" "{classicuo_exe}" -ip {SERVER_IP} -port {SERVER_PORT} -path "{client_path}"

echo.
echo Launching NeuroUO Client...
echo This window will close in 5 seconds.
timeout /t 5 /nobreak > nul
exit
"""
    
    batch_path = launcher_dir / "LaunchNeuroUO.bat"
    with open(batch_path, 'w') as f:
        f.write(batch_content)
    
    print(f"Created launcher: {batch_path}")
    return batch_path

def main():
    print("=" * 50)
    print("         NeuroUO Simple Launcher")
    print("=" * 50)
    print()
    
    # Get launcher directory
    if getattr(sys, 'frozen', False):
        app_path = Path(sys.executable).parent
    else:
        app_path = Path(__file__).parent
    
    classicuo_path = app_path / "ClassicUO"
    client_path = app_path / "UOClient"
    
    # Try to find existing ClassicUO
    classicuo_exe = None
    if classicuo_path.exists():
        classicuo_exe = find_classicuo_exe(classicuo_path)
    
    # Download ClassicUO if not found
    if not classicuo_exe:
        print("ClassicUO not found. Trying to download...")
        
        success = False
        for url in CLASSICUO_URLS:
            print(f"\nTrying: {url}")
            temp_zip = tempfile.mktemp(suffix='.zip')
            
            if download_file(url, temp_zip):
                try:
                    extract_zip(temp_zip, classicuo_path)
                    classicuo_exe = find_classicuo_exe(classicuo_path)
                    
                    if classicuo_exe:
                        print(f"Found ClassicUO at: {classicuo_exe}")
                        success = True
                        break
                except Exception as e:
                    print(f"Extraction failed: {e}")
                    if os.path.exists(temp_zip):
                        os.remove(temp_zip)
        
        if not success:
            print("\nFailed to download ClassicUO!")
            print("Please download ClassicUO manually from:")
            print("https://www.classicuo.eu/")
            input("\nPress Enter to exit...")
            return
    
    # Check if UO Client exists
    if not client_path.exists() or not any(client_path.glob("*.mul")):
        print("\nUO Client files not found.")
        print("Downloading (this will take a while - 928MB)...")
        
        temp_zip = tempfile.mktemp(suffix='.zip')
        if download_file(UO_CLIENT_URL, temp_zip):
            extract_zip(temp_zip, client_path)
        else:
            print("Failed to download UO Client!")
            print("Please download manually from: https://neurouo.fun/downloads/")
            input("\nPress Enter to exit...")
            return
    
    # Create settings
    create_settings(classicuo_exe.parent, client_path)
    
    # Create batch launcher
    batch_launcher = create_batch_launcher(app_path, classicuo_exe, client_path)
    
    # Launch game
    print(f"\nLaunching NeuroUO (Server: {SERVER_IP}:{SERVER_PORT})...")
    
    try:
        # Try batch file first
        if os.name == 'nt':  # Windows
            os.startfile(str(batch_launcher))
        else:
            # Direct launch for non-Windows
            subprocess.Popen([
                str(classicuo_exe),
                "-ip", SERVER_IP,
                "-port", str(SERVER_PORT),
                "-path", str(client_path)
            ], cwd=str(classicuo_exe.parent))
        
        print("\nLaunched successfully!")
        print("Enjoy playing on NeuroUO!")
        
    except Exception as e:
        print(f"\nError launching: {e}")
        print("\nYou can manually launch by running:")
        print(f"  {batch_launcher}")
        input("\nPress Enter to exit...")

if __name__ == "__main__":
    main()